@@ -70,7 +70,7 @@ static GString* Dirname(GString* str){
 // HtmlString
 //------------------------------------------------------------------------
 
-HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
+HtmlString::HtmlString(GfxState *state, double fontSize, double _charspace, HtmlFontAccu* fonts) {
   GfxFont *font;
   double x, y;
 
@@ -82,7 +82,8 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
     state->getFillRGB(&rgb);
     GString *name = state->getFont()->getName();
     if (!name) name = HtmlFont::getDefaultFont(); //new GString("default");
-    HtmlFont hfont=HtmlFont(name, static_cast<int>(fontSize-1), rgb);
+   // HtmlFont hfont=HtmlFont(name, static_cast<int>(fontSize-1),_charspace, rgb);
+    HtmlFont hfont=HtmlFont(name, static_cast<int>(fontSize-1),0.0, rgb);
     fontpos = fonts->AddFont(hfont);
   } else {
     // this means that the PDF file draws text without a current font,
@@ -104,7 +105,9 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
   len = size = 0;
   yxNext = NULL;
   xyNext = NULL;
+  strSize = 0;
   htext=new GString();
+  htext2=new GString();
   dir = textDirUnknown;
 }
 
@@ -112,6 +115,8 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
 HtmlString::~HtmlString() {
   delete text;
   delete htext;
+  delete htext2;
+//  delete strSize;
   gfree(xRight);
 }
 
@@ -131,6 +136,8 @@ void HtmlString::addChar(GfxState *state, double x, double y,
     xMin = x;
   }
   xMax = xRight[len] = x + dx;
+  //xMax = xRight[len] = x;
+  ++strSize;
 //printf("added char: %f %f xright = %f\n", x, dx, x+dx);
   ++len;
 }
@@ -214,7 +221,7 @@ void HtmlPage::updateFont(GfxState *state) {
 }
 
 void HtmlPage::beginString(GfxState *state, GString *s) {
-  curStr = new HtmlString(state, fontSize, fonts);
+  curStr = new HtmlString(state, fontSize,charspace, fonts);
 }
 
 
@@ -230,6 +237,7 @@ void HtmlPage::conv(){
 
      if (tmp->htext) delete tmp->htext; 
      tmp->htext=HtmlFont::simple(h,tmp->text,tmp->len);
+     tmp->htext2=HtmlFont::simple(h,tmp->text,tmp->len);
 
      if (links->inLink(tmp->xMin,tmp->yMin,tmp->xMax,tmp->yMax, linkIndex)){
        tmp->link = links->getLink(linkIndex);
@@ -246,18 +254,19 @@ void HtmlPage::addChar(GfxState *state, double x, double y,
 		       double dx, double dy, 
 			double ox, double oy, Unicode *u, int uLen) {
   double x1, y1, w1, h1, dx2, dy2;
-  int n, i;
+  int n, i, d;
   state->transform(x, y, &x1, &y1);
   n = curStr->len;
+  d = 0;
  
   // check that new character is in the same direction as current string
   // and is not too far away from it before adding 
-  if ((UnicodeMap::getDirection(u[0]) != curStr->dir) || 
+/*  if ((UnicodeMap::getDirection(u[0]) != curStr->dir) || 
      (n > 0 && 
       fabs(x1 - curStr->xRight[n-1]) > 0.1 * (curStr->yMax - curStr->yMin))) {
     endString();
     beginString(state, NULL);
-  }
+  }*/
   state->textTransformDelta(state->getCharSpace() * state->getHorizScaling(),
 			    0, &dx2, &dy2);
   dx -= dx2;
@@ -267,9 +276,30 @@ void HtmlPage::addChar(GfxState *state, double x, double y,
     w1 /= uLen;
     h1 /= uLen;
   }
+/* if (d != 3)
+ {
+ endString();
+ beginString(state, NULL);
+ }
+*/
+
+
+  for (i = 0; i < uLen; ++i) 
+  {
+	if (u[i] == ' ')
+        {
+	    endString();
+	    beginString(state, NULL);
+	} else {
+	    curStr->addChar(state, x1 + i*w1, y1 + i*h1, w1, h1, u[i]);
+        }
+   }
+
+/*
   for (i = 0; i < uLen; ++i) {
     curStr->addChar(state, x1 + i*w1, y1 + i*h1, w1, h1, u[i]);
   }
+*/
 }
 
 void HtmlPage::endString() {
@@ -331,7 +361,12 @@ void HtmlPage::coalesce() {
   double space, horSpace, vertSpace, vertOverlap;
   GBool addSpace, addLineBreak;
   int n, i;
-  double curX, curY;
+  double curX, curY, lastX, lastY;
+  int sSize = 0;      
+  double diff = 0.0;
+  double pxSize = 0.0;
+  double strSize = 0.0;
+  double cspace = 0.0;
 
 #if 0 //~ for debugging
   for (str1 = yxStrings; str1; str1 = str1->yxNext) {
@@ -351,7 +386,7 @@ void HtmlPage::coalesce() {
 
   //----- discard duplicated text (fake boldface, drop shadows)
   if( !complexMode )
-  {	/* if not in complex mode get rid of duplicate strings */
+  {
 	HtmlString *str3;
 	GBool found;
   	while (str1)
@@ -385,21 +420,20 @@ void HtmlPage::coalesce() {
 			str1 = str1->yxNext;
 		}
 	}		
-  }	/*- !complexMode */
+  }
   
   str1 = yxStrings;
   
   hfont1 = getFont(str1);
-  if( hfont1->isBold() )
-    str1->htext->insert(0,"<b>",3);
-  if( hfont1->isItalic() )
-    str1->htext->insert(0,"<i>",3);
+
+  str1->htext2->append(str1->htext);
   if( str1->getLink() != NULL ) {
     GString *ls = str1->getLink()->getLinkStart();
     str1->htext->insert(0, ls);
     delete ls;
   }
   curX = str1->xMin; curY = str1->yMin;
+  lastX = str1->xMin; lastY = str1->yMin;
 
   while (str1 && (str2 = str1->yxNext)) {
     hfont2 = getFont(str2);
@@ -434,15 +468,17 @@ void HtmlPage::coalesce() {
 	 ) ||
        	 (vertSpace >= 0 && vertSpace < 0.5 * space && addLineBreak)
 	) &&
-	(!complexMode || (hfont1->isEqualIgnoreBold(*hfont2))) && // in complex mode fonts must be the same, in other modes fonts do not metter
+ // in complex mode fonts must be the same, in other modes fonts do not metter
 	str1->dir == str2->dir // text direction the same
        ) 
     {
-//      printf("yes\n");
-      n = str1->len + str2->len;
-      if ((addSpace = horSpace > 0.1 * space)) {
+     diff = str2->xMax - str1->xMin;
+
+     n = str1->len + str2->len;
+     if ((addSpace = horSpace > 0.1 * space)) {
         ++n;
       }
+    
       if (addLineBreak) {
         ++n;
       }
@@ -453,14 +489,26 @@ void HtmlPage::coalesce() {
       str1->xRight = (double *)grealloc(str1->xRight,
 					str1->size * sizeof(double));
       if (addSpace) {
-		  str1->text[str1->len] = 0x20;
-		  str1->htext->append(" ");
-		  str1->xRight[str1->len] = str2->xMin;
-		  ++str1->len;
+		/*  if (addSpace > (xoutRoundLower(hfont1->getSize()/scale)))
+		  {
+		  	str1->text[str1->len] = 0x20;
+			str1->htext->append(" ");
+			str1->htext2->append(" ");
+			str1->xRight[str1->len] = str2->xMin;
+			++str1->len;
+			++str1->strSize;
+		 } */
+  	   	 str1->text[str1->len] = 0x20;
+                 str1->htext->append(" ");
+                 str1->htext2->append(" ");
+                 str1->xRight[str1->len] = str2->xMin;
+                 ++str1->len;
+                ++str1->strSize;
       }
       if (addLineBreak) {
 	  str1->text[str1->len] = '\n';
 	  str1->htext->append("<br>");
+	  str1->htext2->append(" ");
 	  str1->xRight[str1->len] = str2->xMin;
 	  ++str1->len;
 	  str1->yMin = str2->yMin;
@@ -468,6 +516,7 @@ void HtmlPage::coalesce() {
 	  str1->xMax = str2->xMax;
 	  int fontLineSize = hfont1->getLineSize();
 	  int curLineSize = (int)(vertSpace + space); 
+
 	  if( curLineSize != fontLineSize )
 	  {
 	      HtmlFont *newfnt = new HtmlFont(*hfont1);
@@ -476,9 +525,24 @@ void HtmlPage::coalesce() {
 	      delete newfnt;
 	      hfont1 = getFont(str1);
 	      // we have to reget hfont2 because it's location could have
-	      // changed on resize
+	      // changed on resize  GStri;ng *iStr=GString::fromInt(i);
 	      hfont2 = getFont(str2); 
 	  }
+
+      }
+      str1->htext2->append(str2->htext2);
+
+      HtmlLink *hlink1 = str1->getLink();
+      HtmlLink *hlink2 = str2->getLink();
+
+      GString *fntFix;
+      GString *iStr=GString::fromInt(str2->fontpos);     
+      fntFix = new GString("</span><span class=\"ft");
+      fntFix->append(iStr);
+      fntFix->append("\">");
+      if (((hlink1 == NULL) && (hlink2 == NULL)) && (hfont1->isEqualIgnoreBold(*hfont2) == gFalse))
+      {
+	str1->htext->append(fntFix);
       }
       for (i = 0; i < str2->len; ++i) {
 	str1->text[str1->len] = str2->text[i];
@@ -486,19 +550,6 @@ void HtmlPage::coalesce() {
 	++str1->len;
       }
 
-      /* fix <i> and <b> if str1 and str2 differ */
-      if( hfont1->isBold() && !hfont2->isBold() )
-	str1->htext->append("</b>", 4);
-      if( hfont1->isItalic() && !hfont2->isItalic() )
-	str1->htext->append("</i>", 4);
-      if( !hfont1->isBold() && hfont2->isBold() )
-	str1->htext->append("<b>", 3);
-      if( !hfont1->isItalic() && hfont2->isItalic() )
-	str1->htext->append("<i>", 3);
-
-      /* now handle switch of links */
-      HtmlLink *hlink1 = str1->getLink();
-      HtmlLink *hlink2 = str2->getLink();
       if( !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2) ) {
 	if(hlink1 != NULL )
 	  str1->htext->append("</a>");
@@ -510,34 +561,54 @@ void HtmlPage::coalesce() {
       }
 
       str1->htext->append(str2->htext);
+      sSize = str1->htext2->getLength();      
+      pxSize = xoutRoundLower(hfont1->getSize()/scale);
+      strSize = (pxSize*(sSize-2));   
+      cspace = (diff / strSize);//(strSize-pxSize));
+     // we check if the fonts are the same and create a new font to ajust the text
+//      double diff = str2->xMin - str1->xMin;
+//      printf("%s\n",str1->htext2->getCString());
       // str1 now contains href for link of str2 (if it is defined)
       str1->link = str2->link; 
+
+      HtmlFont *newfnt = new HtmlFont(*hfont1);
+      newfnt->setCharSpace(cspace);
+      //newfnt->setLineSize(curLineSize);
+      str1->fontpos = fonts->AddFont(*newfnt);
+      delete newfnt;
+      hfont1 = getFont(str1);
+      // we have to reget hfont2 because it's location could have
+      // changed on resize  GStri;ng *iStr=GString::fromInt(i);
+      hfont2 = getFont(str2); 
+
       hfont1 = hfont2;
+
       if (str2->xMax > str1->xMax) {
 	str1->xMax = str2->xMax;
       }
+
       if (str2->yMax > str1->yMax) {
 	str1->yMax = str2->yMax;
       }
+
       str1->yxNext = str2->yxNext;
+
       delete str2;
-    } else { // keep strings separate
+    } else { 
+
+//     printf("startX = %f, endX = %f, diff = %f, fontsize = %d, pxSize = %f, stringSize = %d, cspace = %f, strSize = %f\n",str1->xMin,str1->xMax,diff,hfont1->getSize(),pxSize,sSize,cspace,strSize);
+
+// keep strings separate
 //      printf("no\n"); 
-      if( hfont1->isBold() )
-	str1->htext->append("</b>",4);
-      if( hfont1->isItalic() )
-	str1->htext->append("</i>",4);
+//      if( hfont1->isBold() )
       if(str1->getLink() != NULL )
-	str1->htext->append("</a>");
+	str1->htext->append("</a>");  
      
       str1->xMin = curX; str1->yMin = curY; 
       str1 = str2;
       curX = str1->xMin; curY = str1->yMin;
       hfont1 = hfont2;
-      if( hfont1->isBold() )
-	str1->htext->insert(0,"<b>",3);
-      if( hfont1->isItalic() )
-	str1->htext->insert(0,"<i>",3);
+
       if( str1->getLink() != NULL ) {
 	GString *ls = str1->getLink()->getLinkStart();
 	str1->htext->insert(0, ls);
@@ -546,10 +617,7 @@ void HtmlPage::coalesce() {
     }
   }
   str1->xMin = curX; str1->yMin = curY;
-  if( hfont1->isBold() )
-    str1->htext->append("</b>",4);
-  if( hfont1->isItalic() )
-    str1->htext->append("</i>",4);
+
   if(str1->getLink() != NULL )
     str1->htext->append("</a>");
 
@@ -565,6 +633,7 @@ void HtmlPage::coalesce() {
 
 }
 
+
 void HtmlPage::dumpAsXML(FILE* f,int page){  
   fprintf(f, "<page number=\"%d\" position=\"absolute\"", page);
   fprintf(f," top=\"0\" left=\"0\" height=\"%d\" width=\"%d\">\n", pageHeight,pageWidth);
@@ -757,6 +826,11 @@ void HtmlPage::setDocName(char *fname){
   DocName=new GString(fname);
 }
 
+void HtmlPage::updateCharSpace(GfxState *state)
+{
+	charspace = state->getCharSpace();
+}
+
 //------------------------------------------------------------------------
 // HtmlMetaVar
 //------------------------------------------------------------------------
@@ -865,7 +939,7 @@ HtmlOutputDev::HtmlOutputDev(char *fileName, char *title,
   pages = new HtmlPage(rawOrder, extension);
   
   glMetaVars = new GList();
-  glMetaVars->append(new HtmlMetaVar("generator", "pdftohtml 0.39"));  
+  glMetaVars->append(new HtmlMetaVar("generator", "pdftohtml 0.40"));  
   if( author ) glMetaVars->append(new HtmlMetaVar("author", author));  
   if( keywords ) glMetaVars->append(new HtmlMetaVar("keywords", keywords));  
   if( date ) glMetaVars->append(new HtmlMetaVar("date", date));  
@@ -1060,7 +1134,7 @@ void HtmlOutputDev::endString(GfxState *state) {
 void HtmlOutputDev::drawChar(GfxState *state, double x, double y,
 	      double dx, double dy,
 	      double originX, double originY,
-	      CharCode code, Unicode *u, int uLen) 
+	      CharCode code, int nBytes, Unicode *u, int uLen) 
 {
   if ( !showHidden && (state->getRender() & 3) == 3) {
     return;
@@ -1546,3 +1620,8 @@ GBool HtmlOutputDev::newOutlineLevel(FILE *output, Object *node, Catalog* catalo
 
   return atLeastOne;
 }
+
+void HtmlOutputDev::updateCharSpace(GfxState *state)
+{
+	pages->updateCharSpace(state);
+}
