@@ -46,7 +46,7 @@ GBool xml=gFalse;
 GBool errQuiet=gFalse;
 
 GBool showHidden = gFalse;
-GBool noMerge = gFalse;
+GBool noMerge = gTrue;
 static char ownerPassword[33] = "";
 static char userPassword[33] = "";
 static char gsDevice[33] = "png16m";
@@ -86,8 +86,8 @@ static ArgDesc argDesc[] = {
    "output for XML post-processing"},
   {"-hidden", argFlag,   &showHidden,   0,
    "output hidden text"},
-  {"-nomerge", argFlag, &noMerge, 0,
-   "do not merge paragraphs"},   
+/*  {"-nomerge", argFlag, &noMerge, 0,
+   "do not merge paragraphs"}, */  
   {"-enc",    argString,   textEncName,    sizeof(textEncName),
    "output text encoding name"},
   {"-dev",    argString,   gsDevice,       sizeof(gsDevice),
@@ -120,7 +120,7 @@ int main(int argc, char *argv[]) {
   // parse args
   ok = parseArgs(argDesc, &argc, argv);
   if (!ok || argc < 2 || argc > 3 || printHelp || printVersion) {
-    fprintf(stderr, "pdftohtml version %s http://pdftohtml.sourceforge.net/, based on Xpdf version %s\n", "0.39", xpdfVersion);
+    fprintf(stderr, "pdftohtml version %s http://pdftohtml.sourceforge.net/, based on Xpdf version %s\n", "0.40", xpdfVersion);
     fprintf(stderr, "%s\n", "Copyright 1999-2003 Gueorgui Ovtcharov and Rainer Dorsch");
     fprintf(stderr, "%s\n\n", xpdfCopyright);
     if (!printVersion) {
@@ -287,8 +287,10 @@ int main(int argc, char *argv[]) {
 
   if (htmlOut->isOk())
   {
-	doc->displayPages(htmlOut, firstPage, lastPage, static_cast<int>(72*scale), static_cast<int>(72*scale), 0, gTrue, gTrue);
-  	if (!xml)
+//	doc->displayPages(htmlOut, firstPage, lastPage, static_cast<int>(72*scale), static_cast<int>(72*scale), 0, gTrue, gTrue);
+	doc->displayPages(htmlOut, firstPage, lastPage, static_cast<int>(72*scale), static_cast<int>(72*scale), 0, gTrue, gTrue,gTrue,NULL);
+  	
+	if (!xml)
 	{
 		htmlOut->dumpDocOutline(doc->getCatalog());
 	}
@@ -308,7 +310,8 @@ int main(int argc, char *argv[]) {
     globalParams->setPSNoText(gTrue);
     psOut = new PSOutputDev(psFileName->getCString(), doc->getXRef(),
 			    doc->getCatalog(), firstPage, lastPage, psModePS);
-    doc->displayPages(psOut, firstPage, lastPage, static_cast<int>(72*scale), static_cast<int>(72*scale), 0, gTrue, gTrue);
+   // doc->displayPages(psOut, firstPage, lastPage, static_cast<int>(72*scale), static_cast<int>(72*scale), 0, gTrue, gTrue);
+	doc->displayPages(psOut, firstPage, lastPage, static_cast<int>(72*scale), static_cast<int>(72*scale), 0, gTrue, gTrue,gTrue,NULL);
     delete psOut;
 
     /*sprintf(buf, "%s -sDEVICE=png16m -dBATCH -dNOPROMPT -dNOPAUSE -r72 -sOutputFile=%s%%03d.png -g%dx%d -q %s", GHOSTSCRIPT, htmlFileName->getCString(), w, h,
